<?php

namespace Drupal\abjs\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class for build form condition.
 */
class AbjsConditionForm extends FormBase {

  /**
   * Current account user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $account;

  /**
   * Class constructor.
   */
  public function __construct(AccountInterface $account) {
    $this->account = $account;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'abjs_condition';
  }

  /**
   * Building the form.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The state of forms.
   * @param int $cid
   *   The ID of the item.
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cid = NULL) {
    $form = [];
    $condition_name_default = "";
    $condition_script_default = "";
    if (!empty($cid)) {
      $condition_result = db_query('SELECT name, script FROM {abjs_condition} WHERE cid = :cid', [':cid' => $cid]);
      $condition = $condition_result->fetchObject();
      if (empty($condition)) {
        drupal_set_message($this->t('The requested condition does not exist.'), 'error');
        return $form;
      }
      $condition_name_default = $condition->name;
      $condition_script_default = $condition->script;
      $form['cid'] = ['#type' => 'value', '#value' => $cid];
    }

    $form['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Condition Name'),
      '#default_value' => $condition_name_default,
      '#size' => 30,
      '#maxlength' => 50,
      '#required' => TRUE,
    ];

    $form['script'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Condition Script'),
      '#default_value' => $condition_script_default,
      '#description' => $this->t('Any valid javascript with a return statement at the end, returning true or false. Read the documentation for examples'),
      '#rows' => 3,
      '#required' => TRUE,
    ];
    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['save'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#weight' => 5,
      '#submit' => ['::saveCondition'],
      '#attributes' => ['class' => ["button button-action button--primary"]],
    ];
    $form['actions']['cancel'] = [
      '#type' => 'submit',
      '#value' => $this->t('Cancel'),
      '#weight' => 10,
      '#submit' => ['::cancelCondition'],
      '#limit_validation_errors' => [],
    ];
    if (!empty($cid)) {
      $form['actions']['delete'] = [
        '#type' => 'submit',
        '#value' => $this->t('Delete'),
        '#weight' => 15,
        '#submit' => ['::deleteCondition'],
      ];
    }

    // Add ace code editor for syntax highlighting on the script field.
    if ($this->configFactory()->get('abjs.settings')->get('ace') == 1) {
      $form['#attached']['library'][] = 'abjs/ace-editor';
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {}

  /**
   * Save data.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The state of forms.
   */
  public function saveCondition(array &$form, FormStateInterface $form_state) {
    $user = $this->account;
    if ($form_state->hasValue('cid')) {
      // This is a modified condition, so use update.
      db_update('abjs_condition')
        ->fields([
          'name' => $form_state->getValue('name'),
          'script' => $form_state->getValue('script'),
          'changed' => REQUEST_TIME,
          'changed_by' => $user->id(),
        ])
        ->condition('cid', $form_state->getValue('cid'), '=')
        ->execute();
      drupal_set_message($this->t("Successfully updated condition"));

    }
    else {
      // This is a new condition, so use insert.
      db_insert('abjs_condition')
        ->fields([
          'name' => $form_state->getValue('name'),
          'script' => $form_state->getValue('script'),
          'created' => REQUEST_TIME,
          'created_by' => $user->id(),
          'changed' => REQUEST_TIME,
          'changed_by' => $user->id(),
        ])->execute();
      drupal_set_message($this->t("Successfully saved new condition"));

    }
    $form_state->setRedirect('abjs.condition_admin');
  }

  /**
   * Cancel the action.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The state of forms.
   */
  public function cancelCondition(array &$form, FormStateInterface $form_state) {
    $form_state->setRedirect('abjs.condition_admin');
  }

  /**
   * Delete item.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The state of forms.
   */
  public function deleteCondition(array &$form, FormStateInterface $form_state) {
    $form_state->setRedirect('abjs.condition_delete_confirm_form', ['cid' => $form_state->getValue('cid')]);
  }

}
